/*
 * Copyright  2010 Canonical, Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Authors: Chase Douglas <chase.douglas@canonical.com>
 *
 */

#ifndef _GESTUREPROTO_H_
#define _GESTUREPROTO_H_

#include <stdint.h>
#include <X11/X.h>

#define Window  uint32_t
#define Time    uint32_t

#define X_GestureQueryVersion                1
#define X_GestureSelectEvents                2
#define X_GestureGetSelectedEvents           3

#define GESTUREREQUESTS (X_GestureGetSelectedEvents - X_GestureQueryVersion + 1)

#define GestureAllDevices 0

/**
 * Used to select for events on a given window.
 * Struct is followed by (mask_len * CARD8), with each bit set representing
 * the event mask for the given type. A mask bit represents an event type if
 * (mask == (1 << type)).
 */
typedef struct {
    uint16_t    device_id;       /**< Device id to select for        */
    uint16_t    mask_len;        /**< Length of mask in 4 byte units */
} GestureEventMask;

typedef struct {
    uint8_t     reqType;                /**< Gesture extension major code */
    uint8_t     ReqType;                /**< Always ::X_GestureQueryVersion */
    uint16_t    length;                 /**< Length in 4 byte units */
    uint16_t    major_version;
    uint16_t    minor_version;
} GestureQueryVersionReq;

typedef struct {
    uint8_t     repType;                /**< ::X_Reply */
    uint8_t     RepType;                /**< Always ::X_GestureQueryVersion */
    uint16_t    sequenceNumber;
    uint32_t    length;
    uint16_t    major_version;
    uint16_t    minor_version;
    uint32_t    pad1;
    uint32_t    pad2;
    uint32_t    pad3;
    uint32_t    pad4;
    uint32_t    pad5;
} GestureQueryVersionReply;

typedef struct {
    uint8_t     reqType;                /**< Gesture extension major code */
    uint8_t     ReqType;                /**< Always ::X_GestureSelectEvents */
    uint16_t    length;                 /**< Length in 4 byte units */
    Window      window;
    GestureEventMask mask;
} GestureSelectEventsReq;

typedef struct {
    uint8_t     reqType;                /**< Gesture extension major code */
    uint8_t     ReqType;                /**< Always ::X_GestureGetSelectedEvents */
    uint16_t    length;                 /**< Length in 4 byte units */
    Window      window;
} GestureGetSelectedEventsReq;

typedef struct {
    uint8_t     repType;                /**< Gesture extension major opcode */
    uint8_t     RepType;                /**< Always ::X_GestureGetSelectedEvents */
    uint16_t    sequenceNumber;
    uint32_t    length;
    uint16_t    num_masks;              /**< Number of GestureEventMask structs
                                             trailing the reply */
    uint16_t    pad0;
    uint32_t    pad1;
    uint32_t    pad2;
    uint32_t    pad3;
    uint32_t    pad4;
    uint32_t    pad5;
} GestureGetSelectedEventsReply;

typedef struct
{
    uint8_t     type;                   /**< Always GenericEvent */
    uint8_t     extension;              /**< Gesture extension offset */
    uint16_t    sequenceNumber;         /**< Xevent sequence number */
    uint32_t    length;                 /**< Length in 4 byte uints */
    uint16_t    evtype;                 /**< X generic event type */
    uint16_t    gesture_id;             /**< Unique ID for gesture */
    uint16_t    gesture_type;           /**< Gesture type (zoom, rotate, etc.) */
    uint16_t    device_id;              /**< Device that generated this gesture */
    Time        time;                   /**< Time of gesture event */
    Window      root;                   /**< Root window event occurred on */
    Window      event;                  /**< Window selecting this event for a client */
    Window      child;                  /**< Top-most window of gesture event */
/* +-------- 32 byte boundary --------+ */
    float       focus_x;                /**< Always window coords, 16.16 fixed point */
    float       focus_y;                /**< Relative to event window */
    uint16_t    status;                 /**< Gesture event status */
    uint16_t    num_props;               /**< Number of properties for gesture event */
/* +---- Gesture properties below ----+ */
} GestureEvent;

#undef Window
#undef Time

#endif /* _GESTUREPROTO_H_ */
